"""
Copyright 2014-2015 Arve Kylling

This file is part of IRoPLUM.

IRoPLUM is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

IRoPLUM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with IRoPLUM.  If not, see <http://www.gnu.org/licenses/>.

"""

import matplotlib.pyplot as plt
import numpy as np
import copy as copy
from shapely.geometry import LineString
import glob as glob
import Camera as Camera
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from scipy import interpolate
from  optparse import OptionParser
import difflib
import sys

import contextlib

@contextlib.contextmanager
def printoptions(*args, **kwargs):
    original = np.get_printoptions()
    np.set_printoptions(*args, **kwargs)
    yield 
    np.set_printoptions(**original)

def alloc_matrix2d(W, H):
    """ Pre-allocate a 2D matrix of empty lists. """
    return [ [ [] for i in range(W) ] for j in range(H) ]


def repr_matrix2d(name, matrix):
    lines = ['{} = ['.format(name)]
    rows = []
    for row in range(len(matrix)):
        itemreprs = [repr(matrix[row][col]) for col in range(len(matrix[row]))]
        rows.append('\n    [\n        {}\n    ]'.format(', '.join(itemreprs)))
    lines.append('{}\n]'.format(','.join(rows)))

    return ''.join(lines)

def order_angles_golden_ratio(theta):
    """
    Order angles to reduce the amount of correlated information
    in subsequent projections.

    Parameters
    ----------
    theta : 1D array of floats
        Projection angles in degrees. Duplicate angles are not allowed.

    Returns
    -------
    indices_generator : generator yielding unsigned integers
        The returned generator yields indices into ``theta`` such that
        ``theta[indices]`` gives the approximate golden ratio ordering
        of the projections. In total, ``len(theta)`` indices are yielded.
        All non-negative integers < ``len(theta)`` are yielded exactly once.

    Notes
    -----
    The method used here is that of the golden ratio introduced
    by T. Kohler.

    References
    ----------
    .. [1] Kohler, T. "A projection access scheme for iterative
           reconstruction based on the golden section." Nuclear Science
           Symposium Conference Record, 2004 IEEE. Vol. 6. IEEE, 2004.
    .. [2] Winkelmann, Stefanie, et al. "An optimal radial profile order
           based on the Golden Ratio for time-resolved MRI."
           Medical Imaging, IEEE Transactions on 26.1 (2007): 68-76.
    """
    interval = 180

    def angle_distance(a, b):
        difference = a - b
        return min(abs(difference % interval), abs(difference % -interval))

    remaining = list(np.argsort(theta))   # indices into theta
    # yield an arbitrary angle to start things off
    index = remaining.pop(0)
    angle = theta[index]
    yield index
    # determine subsequent angles using the golden ratio method
    angle_increment = interval * (1 - (np.sqrt(5) - 1) / 2)
    while remaining:
        angle = (angle + angle_increment) % interval
        insert_point = np.searchsorted(theta[remaining], angle)
        index_below = insert_point - 1
        index_above = 0 if insert_point == len(remaining) else insert_point
        distance_below = angle_distance(angle, theta[remaining[index_below]])
        distance_above = angle_distance(angle, theta[remaining[index_above]])
        if distance_below < distance_above:
            yield remaining.pop(index_below)
        else:
            yield remaining.pop(index_above)


def CalcNearness( c_rec, c_real, N=0, verbose=False, debug=False):
    """
    Calculates the nearness between the original density (c_real) and the 
    reconstructed density (c_rec) for a given iteration. 

    N:  N in Eq. 21 of Laepple et al., ACP, 2004.

    """
    if c_rec.shape != c_real.shape:
        print "c_rec.shape != c_rel.shape",  c_rec.shape, c_real.shape
        exit()

    First=False
    if N<=0.0:
        First=True
        # Only include pixels to be reconstructed.
        ind= np.where(c_real>0)
        c_real_avg = np.mean(c_real[ind])
        sum1 =0.0
        k1=0
        k2=0
        for j,i in zip(ind[0],ind[1]):
#            print j,i
            sum1=sum1 + (c_real[j,i]-c_real_avg)*(c_real[j,i]-c_real_avg)
#            if debug:
#                if c_real[j,i]>0:
#                    k1=k1+1
#                    print "sum1", j, i, sum1, c_real_avg, c_real[j,i]

        N = np.sqrt(sum1)
        Nearness=1.0 # By definition

    else:
        sum2 =0.0
        j=0
        k1=0
        k2=0
        while j<c_real.shape[0]:
            i=0
            while i<c_real.shape[1]:
    #            sum1=sum1 + (c_real[j,i]-c_real_avg)*(c_real[j,i]-c_real_avg)
                sum2=sum2 + (c_rec[j,i]-c_real[j,i])*(c_rec[j,i]-c_real[j,i])
#                if debug:
#                    if c_rec[j,i]>0 or c_real[j,i]>0:
#                        k2=k2+1
#                        print "sum2", j, i, sum2, c_rec[j,i], c_real[j,i]
                i=i+1
            j=j+1


        Nearness = np.sqrt(sum2)/N

#    if debug:
#        print "c_real_avg", c_real_avg, N, Nearness, k1, k2
    if verbose:
        print 'Nearness {0:12.4e} {1:12.4e} '.format(Nearness, N)

    if First:
        return Nearness, N
    else:
        return Nearness

def ReadCameraFiles(CameraFiles,RR,verbose=False,plot=False):
#    verbose=True

    Cameras=[]  # List of Cameras
    i=0
    # Read CF and fill info into Camera class
    tmpimage = RR.Image*0
    for CF in CameraFiles:
        if verbose:
            print '#######################################################################'
            print 'CameraFile no. {0:2d} {1:s}: '.format(i, CF)
            print '#######################################################################'
        tmpCamName = 'Cam {0:2d}'.format(i)
        tmpCam = Camera.Camera(CameraFile=CF, Name=tmpCamName)
        tmpCam.Rays()
        tmpCam.Rq = np.zeros(tmpCam.Nrays)
        tmpCam.TotalLength = np.zeros(tmpCam.Nrays)
        iRay=0
        PlotRay=False
        while iRay < tmpCam.Nrays:
            tmpCamRq, tmpCamTotalLength, tmpind, tmpN = tmpCam.CalculateLineIntegral( RR, 
                                                                   iRay, Plot=PlotRay, quiet=True)
            iRay=iRay+1

        tmpimagecovered=tmpCam.PixelsCoveredByRays(RR,plot=False)
        if verbose:
            print "Pixels covered by rays from Camera:", tmpCam.Name
            print tmpimagecovered
        if i==0:
            tmpimage = np.where((tmpimagecovered>0.0),1.0,0.0)
            a = np.array(tmpimage)
            c = np.copy(a)
        else:
            tmpimage = np.where((tmpimagecovered>0.0) & (tmpimage>0.0),1.0,0.0)
            b = np.array(tmpimage)
            c = b*c
        tmpNN = np.where(tmpimage >=1)
#        print "No pixels covered:", len(tmpNN[0])
        Cameras.append(tmpCam)
        i=i+1

    if plot:
        fig, (ax1) = plt.subplots(1, 1, figsize=(4, 4))
        tmpdx = (RR.xmax-RR.xmin)/(RR.nx)
        tmpdz = (RR.zmax-RR.zmin)/(RR.nx)
        z, x = np.mgrid[slice(RR.zmin, RR.zmax+tmpdz/2 , tmpdz),
                        slice(RR.xmin, RR.xmax+tmpdx/2 , tmpdx)]
        z_min, z_max = -np.abs(z).max(), np.abs(z).max()
        z_min, z_max = -np.abs(RR.Image).max(), np.abs(RR.Image).max()
        im=ax1.pcolor(x, z, tmpimage, cmap='RdBu') #, vmin=z_min, vmax=z_max)
        plt.show()

    # Make artifical camera which consists of all rays from single cameras.
    ArtCam = Camera.Camera(Empty=True)

    First=True
    for Cam in Cameras:
        ArtCam.Sinogram = np.concatenate((ArtCam.Sinogram,Cam.Sinogram))
        ArtCam.Angle  = np.concatenate((ArtCam.Angle,Cam.Angle))
        ArtCam.Rq  = np.concatenate((ArtCam.Rq,Cam.Rq))
        ArtCam.TotalLength  = np.concatenate((ArtCam.TotalLength,Cam.TotalLength))
        ArtCam.Nrays = Cam.Nrays
        for tmpind in Cam.ind:
            ArtCam.ind.append(tmpind)      
        ArtCam.a      = np.concatenate((ArtCam.a,Cam.a))
        ArtCam.b      = np.concatenate((ArtCam.b,Cam.b))
        ArtCam.Rays_x = np.hstack((ArtCam.Rays_x,Cam.Rays_x))
        ArtCam.Rays_y = np.hstack((ArtCam.Rays_y,Cam.Rays_y))
    ArtCam.Nrays = len(ArtCam.Angle)
    ArtCam.PixelsCoveredByRaysImage = tmpimage
    ArtCam.PixelsCoveredByAllRays = c
    n = np.where(c>0)
    ArtCam.NumberOfPixelsCoveredByAllRays = len(n[0])
    ArtCam.PixelsCoveredByRaysIndices = np.where(tmpimage>0.0)
    if verbose:
        print "Pixels covered by rays from all Cameras"
        print ArtCam.PixelsCoveredByAllRays

    return ArtCam

def plotImage(Image):
    fig, (ax1) = plt.subplots(1, 1, figsize=(4, 4))
    ax1.imshow( Image, cmap='RdBu') #, vmin=z_min, vmax=z_max)
#    im=ax1.pcolor(x, z, tmpimage1, cmap='RdBu') #, vmin=z_min, vmax=z_max)
    plt.show()

def RunTest(Test,TestDir,InpDir,OutDir, force):
    print '#######  START TEST Reconstruction2D.py #######'
    print 'Running test: {:s}: {:s}'.format(__name__, Test)

    if Test=='Squares_2X2_4Rays_2Projections_RB':
        CameraFiles=InpDir+"Squares_2X2_4Rays_2Projections_RB_Cam[1,2].dat"
        TargetFile =InpDir+"Squares_2X2_4Rays_2Projections_RB.npz"
        RunTest_Squares_2X2_4Rays_2Projections(Test,TestDir,InpDir,OutDir,CameraFiles,force,
                                               TargetFile=TargetFile)
    elif Test=='Squares_2X2_4Rays_2Projections_LB':
        # Use same test as for RB, should work.
        CameraFiles=InpDir+"Squares_2X2_4Rays_2Projections_LB_Cam[1,2].dat"
        TargetFile =InpDir+"Squares_2X2_4Rays_2Projections_LB.npz"
        RunTest_Squares_2X2_4Rays_2Projections(Test,TestDir,InpDir,OutDir,CameraFiles,force,
                                               TargetFile=TargetFile)
    elif Test=='Squares_2X2_6Rays_2Projections_3CamsA':
        # Use same test as for RB, should work.
        CameraFiles=InpDir+"Squares_2X2_6Rays_2Projections_3CamsA_Cam[1,2,3].dat"
        #        TargetFile =InpDir+"Squares_2X2_6Rays_2Projections_3CamsA.npz"
        # TargetFile have different resolution from reconstruction grid and Nearness
        # may thus not be calculated for this test case.
        RunTest_Squares_2X2_6Rays_2Projections_3CamsA(Test,TestDir,InpDir,OutDir,CameraFiles,force)

    print '#######  END TEST Reconstruction2D.py #######'

    return

def RunTest_Squares_2X2_6Rays_2Projections_3CamsA(Test,TestDir,InpDir,OutDir,CameraFiles,force,
                                                  TargetFile=None, verbose=False):
    """
    See SyntheticLineIntegrals.py for description of test.
    """
    verbose=False # True #
    nx=16
    dx=25 #  In meters
    z0=300
    RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=z0 )
    Nq = 30 # Number of iterations

    class args:
        def __init__(self):
            return

    tmpDir=OutDir
    if force:
        fnTestReconstructed     = TestDir+Test+'_Reconst.npz'
        fnTestReconstructedTiff = TestDir+Test+'_Reconst.tiff'
    else:
        fnTestReconstructed     = tmpDir+Test+'_Reconst.npz'
        fnTestReconstructedTiff = tmpDir+Test+'_Reconst.tiff'


    args=args()
    args.CameraFiles=CameraFiles
    args.RRImageFile=fnTestReconstructed
    args.TargetFile=None
    args.TargetImageFile=TargetFile
    args.RRImageFileTiff=fnTestReconstructedTiff

    main(args, Nq=Nq, RR=RR, nx=nx, dx=dx, z0=z0)

    if  not force:
        # Compare this run with old results
        fnold = sorted(glob.glob(fnTestReconstructed))
        for fno in fnold:
            fnn = fno.replace(TestDir,tmpDir)
            print "Comparing files ", fno, " and ", fnn
            s1 = open(fno).readlines()
            s2 = open(fnn).readlines()
            i=0
            for line in difflib.unified_diff(s1, s2, fromfile=fno, tofile=fnn,n=0):
                sys.stdout.write(line)
                i=i+1
            print "Total number of different lines ", i
    return

def RunTest_Squares_2X2_4Rays_2Projections(Test,TestDir,InpDir,OutDir,CameraFiles,force,
                                           TargetFile=None,verbose=False):
    """
    See SyntheticLineIntegrals.py for description of test.
    """
    verbose=False # True #
    nx=4
    dx=100 #  In meters
    z0=300
    RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=z0 )
    Nq = 30 # Number of iterations

    class args:
        def __init__(self):
            return

    tmpDir=OutDir
    if force:
        fnTestReconstructed     = TestDir+Test+'_Reconst.npz'
        fnTestReconstructedTiff = TestDir+Test+'_Reconst.tiff'
    else:
        fnTestReconstructed     = tmpDir+Test+'_Reconst.npz'
        fnTestReconstructedTiff = tmpDir+Test+'_Reconst.tiff'

    args=args()
    args.CameraFiles=CameraFiles
    args.RRImageFile=fnTestReconstructed
    args.TargetFile=None
    args.TargetImageFile=TargetFile
    args.RRImageFileTiff=fnTestReconstructedTiff

    main(args, Nq=Nq, RR=RR, nx=nx, dx=dx, z0=z0)

    if  not force:
        # Compare this run with old results
        fnold = sorted(glob.glob(fnTestReconstructed))
        for fno in fnold:
            fnn = fno.replace(TestDir,tmpDir)
            print "Comparing files ", fno, " and ", fnn
            s1 = open(fno).readlines()
            s2 = open(fnn).readlines()
            i=0
            for line in difflib.unified_diff(s1, s2, fromfile=fno, tofile=fnn,n=0):
                sys.stdout.write(line)
                i=i+1
            print "Total number of different lines ", i
    return

def main(args, Nq=51, RR=None, ColLimit=0.0, nx=160, dx=2.5, z0=300):
         # nx=4, dx=100, z0=300): 
    #  
    """
        Nq : Number of iterations
        RR : Area class

    """

    import Camera as Camera
    verbose=False # True #
    Plot=False #True #
    PlotReconstructionGrid=False #True#
    PlotRay=False #True #
    iPlotRay=6
#    nx=16#4 #10 #100#50 #20 #  200 # 400 # 200 # 501 #
#    dx=25#100 #20 # 2  # 4 #10 #   0.5 #1   # 1. #  In meters
    #RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=350 )
    #RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=400 )
    if RR==None:
        RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=z0 )

    if verbose:
        print "RR", RR.x.min(), RR.x.max(), \
                RR.z.min(), RR.z.max(), \
                RR.Image.shape, RR.x.shape, RR.z.shape
        exit()

    CameraFiles = args.CameraFiles # "LineIntegralGaussCam?.dat"
    CameraFiles = sorted(glob.glob(CameraFiles))
    if verbose:
        print 'CameraFiles:', CameraFiles
    nCameras = len(CameraFiles)
    fnRR = args.RRImageFile

    ### Done with all the input stuff. Now the action.

    if args.TargetImageFile != None:
        Target = Camera.Area(Empty=True)
        Target = Target.read(args.TargetImageFile)
        TargetTot = Target.sumImage()
#        print "TargetTot: ", Target.Image.min(), Target.Image.max(), TargetTot
        if verbose:
            print "Target", Target.Image.min(), Target.Image.max(), \
                Target.x.min(), Target.x.max(), \
                Target.x_mid.min(), Target.x_mid.max(), \
                Target.z.min(), Target.z.max(), \
                Target.Image.shape, Target.x.shape, Target.x_mid.shape, Target.z.shape

        plotimg=False
        if plotimg:
            fig, (ax1) = plt.subplots(1, 1, figsize=(4, 4))
                # Add tmpdz/2 etc to get center of pixel values, not boundaries
#            z, x = np.mgrid[slice(Target.z_mid.min(), Target.z_mid.max()+Target.dz/2 , Target.dz),
            z, x = np.mgrid[slice(Target.z_mid.max(), Target.z_mid.min()-Target.dz/2 , -Target.dz),
                            slice(Target.x_mid.min(), Target.x_mid.max()+Target.dx/2 , Target.dx)]
            #            z_min, z_max = -np.abs(z).max(), np.abs(z).max()
            #            z_min, z_max = -np.abs(Target.Image).max(), np.abs(Target.Image).max()
            im=ax1.pcolor(x, z, Target.Image, cmap='RdBu') #, vmin=z_min, vmax=z_max)
            plt.show()
            exit()

    # Get all camera info from line integral files. 
    # Synthetic line integrals may be generated by SyntheticLineIntegrals.py

    # Read all cameras
    Camera = ReadCameraFiles(CameraFiles,RR)

    if verbose:
        print "Camera.Nrays", Camera.Nrays

    image = np.random.random((nx,nx))
    image = np.zeros((nx,nx)) 
    RR.Image = image

    if PlotRay: 
        xsize=5
        ysize=5
    else:
        xsize=5
        ysize=6
    if Plot:
        fig, (ax1) = plt.subplots(1, 1, figsize=(ysize, xsize))
        fig.subplots_adjust(top=0.90, bottom=0.18, left=0.18, right=0.95)
    else:
        ax1=None

    # Plot reconstruction grid
    if PlotReconstructionGrid:
        color='k'
        for x in RR.x:
            y0 = RR.z[0]
            y1 = RR.z[len(RR.z)-1]
            ax1.plot([x,x],[y0,y1], color=color)
        for y in RR.z:
            x0 = RR.x[0]
            x1 = RR.x[len(RR.z)-1]
            ax1.plot([x0,x1],[y,y], color=color)

        Camera.plot(ax1,step=10)

    # Find which pixels are crossed by which rays.
    tmpiRays=[]
    iRays=[]
#    for iRay in order_angles_golden_ratio(Camera.Angle):
    for iRay in np.arange(len(Camera.Angle)):
        tmpiRays.append(iRay)
    ind=[]
    for iRay in tmpiRays:
#        if PlotRay: 
#            iRay=5            
#        print Camera.Sinogram[iRay]
        iRays.append(iRay)            
        tmpind = Camera.ind[iRay]
        ind.append(tmpind)

    iRayinPixels = alloc_matrix2d(RR.nx, RR.nz)
#    print iRayinPixels
#    print "Camera.PixelsCoveredByAllRays before ColLimit"
#    tmpp = np.where(Camera.PixelsCoveredByAllRays > 0)
#    print len(tmpp[0])

    # This loop is a time killer...........

    i=0
    while i<RR.nx:
        j=0
        while j<RR.nz:
            # Find iRays for this pixel
            for iRay in iRays: #order_angles_golden_ratio(Camera.Angle):
#                for l,k in zip(ind[iRay][0],ind[iRay][1]):
                # FIXME
                for k,l in zip(ind[iRay][0],ind[iRay][1]):
                    if (k==j and l==i):
                        (iRayinPixels[j][i]).append(iRay)
    #                    print i,j, iRay, ind[iRay], Camera.Angle[iRay]
            j=j+1
        i=i+1

    i=0
    while i<RR.nx:
        j=0
        while j<RR.nz:
            # Find iRays for this pixel
            iNotCovered=0
            for iRay in iRayinPixels[j][i]:
                if Camera.Sinogram[iRays[iRay]] <= ColLimit:
                    iNotCovered=iNotCovered+1
                if iNotCovered>=1: #len(iRayinPixels[j][i]):
                    Camera.PixelsCoveredByAllRays[j,i]=0
            j=j+1
        i=i+1


#    print "Camera.PixelsCoveredByAllRays after ColLimit"
    tmpp = np.where(Camera.PixelsCoveredByAllRays > 0)
#    print len(tmpp[0])
#    print Camera.PixelsCoveredByAllRays
#    print tmpCov
#    exit()

    # Inital guess for density field set to zero
    TotalDensity = Camera.TotalDensity()/((Camera.Nrays/len(CameraFiles))*RR.dx)
    InitialGuess = TotalDensity/Camera.NumberOfPixelsCoveredByAllRays
    i=0
    while i<RR.nx:
        j=0
        while j<RR.nz:
            if Camera.PixelsCoveredByAllRays[j][i]>=1:
                RR.Image[j,i] = InitialGuess
            j=j+1
        i=i+1

#    RR.Image=RR.Image*0.0
#    RR.Image[1,1] = 1
#    RR.Image[1,2] = 2
#    RR.Image[2,1] = 3
#    RR.Image[2,2] = 4

#    print "#################################################################"
#    print "Initial image"
#    print "#################################################################"
#    print RR.Image
#    print "Initial guess", RR.Image.max(), InitialGuess, TotalDensity, nx*nx, len(Camera.PixelsCoveredByRaysIndices[0]), len(CameraFiles), Camera.NumberOfPixelsCoveredByAllRays, Camera.Nrays
    RRImage0 = np.copy(RR.Image)
    RRNew    = np.copy(RRImage0)*0.0
    RROld    = np.copy(RRImage0)
    IndZeros = np.copy(RRImage0)*0.0+1  # All pixels that have nonzero sinogram, 
                                    # a value of zero means this pixel has 
                                    # zero density.

#    exit()

    if args.TargetImageFile != None:
        Nearness = np.ones(Nq+1)
        Nearness[0], NearnessN = CalcNearness(RR.Image, Target.Image, verbose=False, debug=False)
#        print "Nearness", 0,  Nearness[0], NearnessN, RR.sumImage()
#        exit()
#    if PlotRay:
#        RR.Image = RR.Image*0.0
        # Plot all rays in black (ray 0) and rest in camera colours
#        Camera.plot(ax1)

    # In Reconstrcution include only Rays with TotalColumn larger than zero 
    Rq = []#np.zeros(Camera.Nrays)
    TotalLength = []#np.zeros(Camera.Nrays)
#    ind=[]
    for iRay in tmpiRays:
        if iRay==0:
            debug=False #True #
        else:
            debug=False #True #
        tmpRq, tmpTotalLength, tmpind, tmpN =Camera.CalculateLineIntegral(RR, iRay, Plot=PlotRay, quiet=True, 
                                                                    debug=debug, ax1=ax1, iPlotRay=iPlotRay)
        if tmpRq > 0.0:
            iRays.append(iRay)            
            Rq.append(tmpRq)
            TotalLength.append(tmpTotalLength)
#            ind.append(tmpind)


    iq=0
    while iq < Nq:
        # Here start loop over all rays.
#        print '#### iq ####', iq

        Rq = np.zeros(Camera.Nrays)
        TotalLength = np.zeros(Camera.Nrays)
        NPix = np.zeros(Camera.Nrays)
#        ind=[]
        for iRay in iRays:
#            print iRay, Camera.Angle[iRay]
#            if PlotRay: 
#                iRay=5
            Rq[iRay], TotalLength[iRay], tmpind, NPix[iRay] =Camera.CalculateLineIntegral(RR, iRays[iRay], Plot=False, quiet=True)
    #                                                                   iRay, Plot=PlotRay, quiet=True)
#            print '{:3d} {:7.4f} {:7.4f} {:7.4f} {:7.4f}'.format( iRay, Rq[iRay], 
#                                                                  Camera.Sinogram[iRay], Camera.Sinogram[iRays[iRay]],
#                                                                  Camera.Angle[iRay]) #tmpind

        RRNew    = np.copy(RRImage0)*0.0
        i=0
        while i<RR.nx:
            j=0
            while j<RR.nz:
                if Camera.PixelsCoveredByAllRays[j][i]>=1:
                    corrN=0
                    sum1=0
                    sum2=0
                    sum3=0
                    sum4=0
                    for iRay in iRayinPixels[j][i]:
                        #
                        #  Eq. 7 in Gilbert (1972) 
                        #
                        sum1=sum1+Camera.Sinogram[iRay]
                        sum2=sum2+TotalLength[iRay]
                        sum3=sum3+Rq[iRay]
#                        N = tmpN #len(tmpind[0])
                        sum4=sum4+NPix[iRay]
#                        corrN=sum1/sum2 - sum3/sum4
                        corrN= (sum1/sum2 - sum3/sum2)
#                        print 'corrN {:2d} {:2d} {:3d} {:7.3f} {:7.4f} {:7.4f} {:7.4f} {:7.4f} {:7.4f} {:7.4f} {:7.4f}'.format( 
#                            i,j,iRay, Camera.Angle[iRay], Rq[iRay], Camera.Sinogram[iRay], TotalLength[iRay],
#                            sum1, sum2, sum3, sum4)
#                        print 'corrN', corrN, iRay, iRays[iRay], Rq[iRay],  Rq[iRays[iRay]], Camera.Sinogram[iRays[iRay]], TotalLength[iRay],  N,sum1/sum2, sum3/sum4, Camera.Angle[iRays[iRay]], Camera.Angle[iRay]
#                    print 'corr', corrN, iRay, RROld[j,i] , RROld[j,i] + corrN
#FIXME
                    RRNew[j,i] = np.maximum(RROld[j,i] + corrN, 0 )
#                    RRNew[i,j] = np.maximum(RROld[i,j] + corrN, 0 )
                j=j+1
            i=i+1

#        print "RRNew", RRNew.max()
#        with printoptions(precision=3, suppress=True, linewidth=100):
#            print(RRNew)
#        print RRNew
        if args.TargetImageFile != None:
            Nearness[iq+1] = CalcNearness(RRNew, Target.Image, N=NearnessN, verbose=False)
#            print "Nearness", iq+1, Nearness[iq+1]

        RROld    = np.copy(RRNew)
        RR.Image = np.copy(RRNew)

        iq=iq+1

    x=RR.x_mid
    z=RR.z_mid
#    print "Save to file", fnRR
    if args.TargetImageFile != None:
        RR.Nearness=Nearness
    RR.save(fnRR)
    if args.RRImageFileTiff != None:
        RR.WriteReconstructedTiff(args.RRImageFileTiff)
    #np.savez(fnRR, x=x, z=z, image=RR.Image)

#    exit()

    # Plot reconstruction pixels
    # generate 2 2d grids for the x & y bounds
    tmpdx = (RR.xmax-RR.xmin)/(nx)
    tmpdz = (RR.zmax-RR.zmin)/(nx)
#    z, x = np.mgrid[slice(RR.zmin, RR.zmax+tmpdz/2 , tmpdz),
    z, x = np.mgrid[slice(RR.zmax, RR.zmin-tmpdz/2 , -tmpdz),
                    slice(RR.xmin, RR.xmax+tmpdx/2 , tmpdx)]
    z_min, z_max = -np.abs(z).max(), np.abs(z).max()
    z_min, z_max = -np.abs(RR.Image).max(), np.abs(RR.Image).max()
    if Plot:
        im=ax1.pcolor(x, z, RR.Image, cmap='RdBu', alpha=0.5) #, vmin=z_min, vmax=z_max)
        margin=50
        ax1.axis([RR.xmin-margin, RR.xmax+margin, RR.zmin-margin, RR.zmax+margin])
    if PlotRay:
        ax1.set_xlabel("Distance (m)")
        ax1.set_ylabel("Heigth (m)")
        xmajorLocator   = MultipleLocator(100)
        xminorLocator   = MultipleLocator(10)
        ymajorLocator   = MultipleLocator(100)
        yminorLocator   = MultipleLocator(10)
        ax1.xaxis.set_major_locator(xmajorLocator)
        ax1.yaxis.set_major_locator(ymajorLocator)
        ax1.xaxis.set_minor_locator(xminorLocator)
        ax1.yaxis.set_minor_locator(yminorLocator)

    elif Plot:
        cb = plt.colorbar(im)

    if Plot:
        plt.show()
#        exit()

    return




############################################################


if __name__ == "__main__":

    RR =  Camera.Area(nx=4,dx=10, x0=0, nz=4, dz=10, z0=300 )

    parser = OptionParser(usage='Reconstruct image from camera line integrals.')
    parser.add_option('--CameraFiles', dest='CameraFiles', default=None,
                      help="""Name (including wild cards) of Camera line integral 
                              input files. Should be one file per camera.""") 
    parser.add_option('--RRImageFile', dest='RRImageFile', default='tmpRRImage',
                      help="""Name of file holding Reconstructed Image.""") 
    parser.add_option('--RRImageFileTiff', dest='RRImageFileTiff', default=None,
                      help="""Name of file holding Reconstructed Image in tiff.""") 
    parser.add_option('--TargetImageFile', dest='TargetImageFile', default=None,
                      help="""Name of Targe Image File.""") 
    (args,rest)   =  parser.parse_args()

    main(args)

