"""
Copyright 2014-2015 Arve Kylling

This file is part of IRoPLUM.

IRoPLUM is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

IRoPLUM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with IRoPLUM.  If not, see <http://www.gnu.org/licenses/>.

"""

import numpy as np
import glob as glob
import sys
import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from  optparse import OptionParser
import Camera as Camera
import difflib

def makeDish(size, radius=1, center=None, rand=False):
    """ Make a dish.
    size is the length of a side of the square
    """
    x = np.arange(0, size, 1, float)
    y = x[:,np.newaxis]
    if center is None:
        x0 = y0 = size // 2
    else:
        x0 = center[0]
        y0 = center[1]

    if rand:
        image = np.random.rand(size,size)
        image = np.where(np.sqrt((x-x0)**2+(y-y0)**2)>radius, 0, image)
    else:
        image = np.where(np.sqrt((x-x0)**2+(y-y0)**2)>radius, 0.0, 1.0)
    return image

def makeEllipse(size, a=1, b=1, center=None, rand=False):
    """ Make an ellipse.
    size is the length of a side of the square
    """
    x = np.arange(0, size, 1, float)
    y = x[:,np.newaxis]
    if center is None:
        x0 = y0 = size // 2
    else:
        x0 = center[0]
        y0 = center[1]

    if rand:
        image = np.random.rand(size,size)
        image = np.where(np.sqrt(((x-x0)/a)**2+((y-y0)/b)**2)>1., 0, image)
    else:
        image = np.where(np.sqrt(((x-x0)/a)**2+((y-y0)/b)**2)>1., 0.0, 1.0)
    return image


def makeGaussian(size, fwhm = 3, center=None):
    """ Make a square gaussian kernel.

    size is the length of a side of the square
    fwhm is full-width-half-maximum, which
    can be thought of as an effective radius.
    """

    x = np.arange(0, size, 1, float)
    y = x[:,np.newaxis]

    if center is None:
        x0 = y0 = size // 2
    else:
        x0 = center[1]
        y0 = center[0]

    return np.exp(-4*np.log(2) * ((x-x0)**2 + (y-y0)**2) / fwhm**2)


def RunTest(Test,TestDir, OutDir, force):
    print '#######  START TEST SyntheticLineIntegrals.py #######'
    print 'Running test: {:s}: {:s}'.format(__name__, Test)

    if Test=='Squares_2X2_4Rays_2Projections_RB':
        RunTest_Squares_2X2_4Rays_2Projections_RB(Test,TestDir, OutDir,force)
    elif Test=='Squares_2X2_4Rays_2Projections_LB':
        RunTest_Squares_2X2_4Rays_2Projections_LB(Test,TestDir, OutDir, force)
    elif Test=='Squares_2X2_6Rays_2Projections_3CamsA':
        RunTest_Squares_2X2_6Rays_2Projections_3CamsA(Test,TestDir, OutDir,force)

    print '#######  END TEST SyntheticLineIntegrals.py #######'
    return

def RunTest_Squares_2X2_6Rays_2Projections_3CamsA(Test,TestDir,OutDir, force,verbose=False):
    """
    Create very simple 4X4 images viewed by three cameras (Projections)
    
          [[ 0.  0.  0.  0.]
           [ 0.  1.  2.  0.]
           [ 0.  3.  4.  0.]
           [ 0.  0.  0.  0.]]

    """

    tmpDir=OutDir
    if force:
        fnTestTarget =       TestDir+Test+'.npz'
        fnTestLineIntegral = TestDir+Test+'_'
    else:
        fnTestTarget = tmpDir+Test+'.npz'
        fnTestLineIntegral = tmpDir+Test+'_'

    Nrays = 24
    nx=4
    dx=100. #  In meters
    z0=300
    FOV=20  # Field of view
    # RR=ReconstructionRegion
    RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=z0 )
    # Shift zmin and zmax with z0
    RR.zmin = RR.z0
    RR.zmax = RR.z0 + RR.dz*RR.nz
    RR.Image = np.zeros([nx,nx])
    RR.Image[1,1] = 1
    RR.Image[1,2] = 2
    RR.Image[2,1] = 3
    RR.Image[2,2] = 4
    if verbose:
        print RR.Image
    # Save Target image. To be used by Compare.py
    if verbose:
        print fnTestTarget
    RR.save(fnTestTarget)

    theta1 = 27.5 - FOV/2
    theta2 = 27.5 + FOV/2
    Camera1=Camera.Camera(x=-1000, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 1')
    Camera1.Rays()
    iRay=0
    while iRay<Camera1.Nrays:
        tmpRq, tmpTotalLength, tmpind, tmpN =Camera1.CalculateLineIntegral(RR, iRay)
        Camera1.Sinogram[iRay]=tmpRq
        iRay=iRay+1
    tmpfn = fnTestLineIntegral+'Cam1.dat'
    if verbose:
        print 'tmpfn', tmpfn
    Camera1.WriteLineIntegralToFile(tmpfn)

    theta1 = 158 - FOV/2
    theta2 = 158 + FOV/2
    Camera2=Camera.Camera(x=1000, z=100, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 2')
    Camera2.Rays()
    iRay=0
    while iRay<Camera2.Nrays: #1: #
        tmpRq, tmpTotalLength, tmpind , tmpN =Camera2.CalculateLineIntegral(RR, iRay)
        Camera2.Sinogram[iRay]=tmpRq
        iRay=iRay+1
    tmpfn = fnTestLineIntegral+'Cam2.dat'
    if verbose:
        print 'tmpfn', tmpfn
    Camera2.WriteLineIntegralToFile(tmpfn)

    FOV=30
    theta1 = 90 - FOV/2
    theta2 = 90 + FOV/2
    Camera3=Camera.Camera(x=0, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 3')
    Camera3.Rays()
    iRay=0
    while iRay<Camera3.Nrays:
        tmpRq, tmpTotalLength, tmpind, tmpN =Camera3.CalculateLineIntegral(RR, iRay)
        Camera3.Sinogram[iRay]=tmpRq
        iRay=iRay+1
    tmpfn = fnTestLineIntegral+'Cam3.dat'
    if verbose:
        print 'tmpfn', tmpfn
    Camera3.WriteLineIntegralToFile(tmpfn)

    if  not force:
        # Compare this run with old results
        fnold = sorted(glob.glob(fnTestLineIntegral+'Cam*.dat'))
        for fno in fnold:
            fnn = fno.replace(TestDir,tmpDir)
            print "Comparing files ", fno, " and ", fnn            
            s1 = open(fno).readlines()
            s2 = open(fnn).readlines()
            i=0
            for line in difflib.unified_diff(s1, s2, fromfile=fno, tofile=fnn,n=0):
                sys.stdout.write(line)
                i=i+1
            print "Total number of different lines ", i
    return

def RunTest_Squares_2X2_4Rays_2Projections_LB(Test,TestDir,OutDir,force,verbose=False):
    """
    Create very simple 4X4 images viewed by two cameras (Projections),
    each with 2 pixels (Nrays).
    
                  [[ 0.  0.  0.  0.]
           --->    [ 0.  1.  2.  0.]
   Cam1    --->    [ 0.  3.  4.  0.]
                   [ 0.  0.  0.  0.]]
        
                         ^   ^
                         |   |

                         Cam 2
    """

    fnTarget = TestDir+Test+'.npz'
    fnLineIntegral = TestDir+Test+'_'
    tmpDir='../tmp/'
    if force:
        fnTestTarget = fnTarget
        fnTestLineIntegral = fnLineIntegral
    else:
        fnTestTarget = tmpDir+Test+'.npz'
        fnTestLineIntegral = tmpDir+Test+'_'

    Nrays = 2 
    nx=4
    dx=100. #  In meters
    z0=300
    FOV=2  # Field of view
    # RR=ReconstructionRegion
    RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=z0 )
    # Shift zmin and zmax with z0
    RR.zmin = RR.z0
    RR.zmax = RR.z0 + RR.dz*RR.nz
    RR.Image = np.zeros([nx,nx])
    RR.Image[1,1] = 1
    RR.Image[1,2] = 2
    RR.Image[2,1] = 3
    RR.Image[2,2] = 4
    if verbose:
        print RR.Image
    # Save Target image. To be used by Compare.py
    if verbose:
        print fnTestTarget
    RR.save(fnTestTarget)

    theta1 = 0 - FOV/2
    theta2 = 0 + FOV/2
    Camera1=Camera.Camera(x=-500, z=500, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 1')
    Camera1.Rays()
    iRay=0
    while iRay<Camera1.Nrays:
        tmpRq, tmpTotalLength, tmpind, tmpN =Camera1.CalculateLineIntegral(RR, iRay)
        Camera1.Sinogram[iRay]=tmpRq
        iRay=iRay+1
    tmpfn = fnTestLineIntegral+'Cam1.dat'
    if verbose:
        print 'tmpfn', tmpfn
    Camera1.WriteLineIntegralToFile(tmpfn)

    theta1 = 90 - FOV/2
    theta2 = 90 + FOV/2
    Camera2=Camera.Camera(x=0, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 2')
    Camera2.Rays()
    iRay=0
    while iRay<Camera2.Nrays:
        tmpRq, tmpTotalLength, tmpind, tmpN =Camera2.CalculateLineIntegral(RR, iRay)
        Camera2.Sinogram[iRay]=tmpRq
        iRay=iRay+1
    tmpfn = fnTestLineIntegral+'Cam2.dat'
    Camera2.WriteLineIntegralToFile(tmpfn)

    if  not force:
        # Compare this run with old results
        fnold = sorted(glob.glob(fnLineIntegral+'Cam*.dat'))
        for fno in fnold:
            fnn = fno.replace(TestDir,tmpDir)
            print "Comparing files ", fno, " and ", fnn            
            s1 = open(fno).readlines()
            s2 = open(fnn).readlines()
            i=0
            for line in difflib.unified_diff(s1, s2, fromfile=fno, tofile=fnn,n=0):
                sys.stdout.write(line)
                i=i+1
            print "Total number of different lines ", i
    return

def RunTest_Squares_2X2_4Rays_2Projections_RB(Test,TestDir,OutDir,force,verbose=False):
    """
    Create very simple 4X4 images viewed by two cameras (Projections),
    each with 2 pixels (Nrays).
    
    [[ 0.  0.  0.  0.]
    [ 0.  1.  2.  0.]       <---
    [ 0.  3.  4.  0.]       <---  Cam 1
    [ 0.  0.  0.  0.]]
        
          ^   ^
          |   |

          Cam 2
    """
    verbose=True

    fnTarget = TestDir+Test+'.npz'
    fnLineIntegral = TestDir+Test+'_'
    tmpDir='../tmp/'
    if force:
        fnTestTarget = fnTarget
        fnTestLineIntegral = fnLineIntegral
    else:
        fnTestTarget = tmpDir+Test+'.npz'
        fnTestLineIntegral = tmpDir+Test+'_'

    Nrays = 2 
    nx=4
    dx=100. #  In meters
    z0=300
    FOV=2  # Field of view
    # RR=ReconstructionRegion
    RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=z0 )
    # Shift zmin and zmax with z0
    RR.zmin = RR.z0
    RR.zmax = RR.z0 + RR.dz*RR.nz
    RR.Image = np.zeros([nx,nx])
    RR.Image[1,1] = 1
    RR.Image[1,2] = 2
    RR.Image[2,1] = 3
    RR.Image[2,2] = 4
    if verbose:
        print RR.Image
    # Save Target image. To be used by Compare.py
    if verbose:
        print fnTestTarget
    RR.save(fnTestTarget)

    theta1 = 180 - FOV/2
    theta2 = 180 + FOV/2
    Camera1=Camera.Camera(x=500, z=500, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 1')
    Camera1.Rays()
    iRay=0
    while iRay<Camera1.Nrays:
        tmpRq, tmpTotalLength, tmpind, tmpN =Camera1.CalculateLineIntegral(RR, iRay)
        Camera1.Sinogram[iRay]=tmpRq
        iRay=iRay+1
    tmpfn = fnTestLineIntegral+'Cam1.dat'
    if verbose:
        print 'tmpfn', tmpfn
    Camera1.WriteLineIntegralToFile(tmpfn)

    theta1 = 90 - FOV/2
    theta2 = 90 + FOV/2
    Camera2=Camera.Camera(x=0, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 2')
    Camera2.Rays()
    iRay=0
    while iRay<Camera2.Nrays:
        tmpRq, tmpTotalLength, tmpind, tmpN =Camera2.CalculateLineIntegral(RR, iRay)
        Camera2.Sinogram[iRay]=tmpRq
        iRay=iRay+1
    tmpfn = fnTestLineIntegral+'Cam2.dat'
    Camera2.WriteLineIntegralToFile(tmpfn)

    if  not force:
        # Compare this run with old results
        fnold = sorted(glob.glob(fnLineIntegral+'Cam*.dat'))
        for fno in fnold:
            fnn = fno.replace(TestDir,tmpDir)
            print "Comparing files ", fno, " and ", fnn            
            s1 = open(fno).readlines()
            s2 = open(fnn).readlines()
            i=0
            for line in difflib.unified_diff(s1, s2, fromfile=fno, tofile=fnn,n=0):
                sys.stdout.write(line)
                i=i+1
            print "Total number of different lines ", i
    return

#########################################################



if __name__ == "__main__":

    """
    Generate synthetic line integrals as recorded by a number
    of cameras. The cameras have one column of pixels in the 
    vertical.

    Different viewing targets are specified with the Target 
    variable.
    """
    PlotType='Sinogram' #'CameraPosition' #  None # 'Target'# 'Ray' #

    Target ='Gauss' #  'Squares' # 'Dish' # 'Ellipse' # 
    PlotTarget = False #True # 
    OutDir='' # '../Examples/'
    fnName = 'tmpTest' # '_5Cameras_201Rays'
    fnLineIntegral = OutDir+'LineIntegral'+Target+fnName
    fnTarget = OutDir+'TargetImage'+Target+fnName+'.pkl'

    Cam1=True # False # 
    Cam2=True # False #
    Cam3=True # False #
    Cam4=True # False #
    Cam5=True # False #
    NCameras=0
    if Cam1: NCameras=NCameras+1
    if Cam2: NCameras=NCameras+1
    if Cam3: NCameras=NCameras+1
    if Cam4: NCameras=NCameras+1
    if Cam5: NCameras=NCameras+1

    Nrays =  201 #24 #6 #2 #  501 #11 #1024 #
    if PlotType=='Target' or PlotType=='CameraPosition':
        Nrays=6 #11
    FOV = 20 #11.0 # Vertical Field of view in degrees.

    # Domain includes reconstruction region and cameras
    Domain = Camera.Area(nx=6,dx=500.,x0=0, nz=10, dz=100., z0=0 )

    nx=160 #500 #50 #51 #
    dx=2.5 #1. #10. #  In meters
    z0=300. #250.
    if Target == 'Squares':
        nx=4
        dx=100. #  In meters
        z0=300
        FOV=18

    # RR=ReconstructionRegion
    RR =  Camera.Area(nx=nx,dx=dx, x0=0, nz=nx, dz=dx, z0=z0 )
    # Shift zmin and zmax with z0
    RR.zmin = RR.z0
    RR.zmax = RR.z0 + RR.dz*RR.nz

    if Target =='Dish':
        radius = 50. # radius of dish in meters
        RR.Image = makeDish(RR.nx,radius=radius/RR.dx)

        radius =  20./dx # radius of dish in meters
        center = np.array([RR.x_center_ind+25/dx,RR.z_center_ind-0/dx])
        image1 = makeDish(RR.nx,radius=radius/RR.dx,center = center )
        radius =   5./dx # radius of dish in meters
        center = np.array([RR.x_center_ind-25/dx,RR.z_center_ind-0/dx])
        image2 = makeDish(RR.nx,radius=radius/RR.dx,center = center  )
        RR.Image = +1.*image1+1.*image2

    elif Target =='Ellipse':
        radius = 50. # radius of dish in meters
        RR.Image = makeEllipse(RR.nx,a=0.75*radius/RR.dx, b=1.5*radius/RR.dx)#, rand=True)
    elif Target =='Gauss':
        zerolimit = 0.01
        fwhm = 25/dx  # In meters
        center = np.array([RR.z_center_ind-10/dx, RR.x_center_ind-15/dx])
        image = makeGaussian(RR.nx, fwhm = fwhm, center=center)
        image1 = np.where(image<zerolimit, 0, image)

        fwhm =  30/dx
        center = np.array([RR.z_center_ind+9/dx, RR.x_center_ind+25/dx])
        image = makeGaussian(RR.nx, fwhm = fwhm, center=center)
        image2 = np.where(image<zerolimit, 0, image)
        RR.Image = +1.*image1+1.5*image2
    elif Target =='Squares':
        RR.Image = np.zeros([nx,nx])
        print RR.Image.shape
        RR.Image[1,1] = 1
        RR.Image[1,2] = 2
        RR.Image[2,1] = 3
        RR.Image[2,2] = 4


    # Save Target image. To be used by Compare.py
    print fnTarget
    RR.save(fnTarget)

    if PlotTarget:
        fig, (ax1) = plt.subplots(1, 1, figsize=(4, 4))
        fig.subplots_adjust(bottom=0.13)
        ax1.imshow(RR.Image, cmap=plt.cm.jet, aspect='auto',
                   interpolation='none')
        plt.show()
        exit()

    if PlotType=='Ray':
        xsize=5
        ysize=5
        fig, (ax1) = plt.subplots(1, 1, figsize=(ysize, xsize))
        fig.subplots_adjust(top=0.90, bottom=0.18, left=0.18, right=0.95)
        # Plot reconstruction grid
        color='k'
        for x in RR.x:
            y0 = RR.z[0]
            y1 = RR.z[len(RR.z)-1]
            ax1.plot([x,x],[y0,y1], color=color)
        for y in RR.z:
            x0 = RR.x[0]
            x1 = RR.x[len(RR.z)-1]
            ax1.plot([x0,x1],[y,y], color=color)
        tmpdx = (RR.xmax-RR.xmin)/(nx)
        tmpdz = (RR.zmax-RR.zmin)/(nx)
        z, x = np.mgrid[slice(RR.zmax ,RR.zmin-tmpdz/2,  -tmpdz),
                        slice(RR.xmin, RR.xmax+tmpdx/2 , tmpdx)]
        im=ax1.pcolor(x, z, RR.Image, cmap='RdBu', alpha=0.5) #, vmin=z_min, vmax=z_max)
        margin=50
        ax1.axis([RR.xmin-margin, RR.xmax+margin, RR.zmin-margin, RR.zmax+margin])

    else:
        ax1=None

    if Cam1:
        theta1 = 27.5 - FOV/2
        theta2 = 27.5 + FOV/2
        if Target =='Squaress':
            theta1 = 45 - FOV/2 #180 - FOV/2
            theta2 = 45 + FOV/2 #180 + FOV/2
            Camera1=Camera.Camera(x=-500, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 1')
#            Camera1=Camera.Camera(x=500, z=500, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 1')
        else:
            Camera1=Camera.Camera(x=-1000, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 1')
    #    Camera1=Camera(x=-500, z=0, theta1=30, theta2=60, Nrays=11,Name='West')
        Camera1.PlotProp.text_x=-1150
        Camera1.PlotProp.text_y= 50
        Camera1.Rays()
#        Camera1.plot(ax1)
#        Camera1.LineIntegral(RR.Image, RR, plot=False)#True)#
        iRay=0
        while iRay<Camera1.Nrays:
            tmpRq, tmpTotalLength, tmpind, tmpN =Camera1.CalculateLineIntegral(RR, iRay)
            Camera1.Sinogram[iRay]=tmpRq
            iRay=iRay+1
        tmpfn = fnLineIntegral+'Cam1.dat'
        print 'tmpfn', tmpfn
        Camera1.WriteLineIntegralToFile(tmpfn)
    #    exit()

    if Cam2:
        theta1 = 158 - FOV/2
        theta2 = 158 + FOV/2
        if Target =='Squares_test':
            theta1 = 90 - FOV/2
            theta2 = 90 + FOV/2
            Camera2=Camera.Camera(x=0, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 2')
        else:
            Camera2=Camera.Camera(x=1000, z=100, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 2')
        Camera2.PlotProp.color='b'
        Camera2.PlotProp.text_x= 950
        Camera2.PlotProp.text_y=  30
        Camera2.Rays()
#        Camera2.plot(ax1)
#        Camera2.LineIntegral(RR.Image, RR, plot=False)
        iRay=0
        while iRay<Camera2.Nrays: #1: #
            tmpRq, tmpTotalLength, tmpind, tmpN =Camera2.CalculateLineIntegral(RR, iRay)#, Plot=True, ax1=ax1, iPlotRay=1, debug=False)
#            print iRay, tmpRq
            Camera2.Sinogram[iRay]=tmpRq
            iRay=iRay+1
        tmpfn = fnLineIntegral+'Cam2.dat'
        print 'tmpfn', tmpfn
        Camera2.WriteLineIntegralToFile(tmpfn)

    if Cam3:
#        FOV=30
        theta1 = 90 - FOV/2
        theta2 = 90 + FOV/2
    #    Camera3=Camera(x=0, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 3')
        Camera3=Camera.Camera(x=0, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 3')
        Camera3.PlotProp.color='g'
        Camera3.PlotProp.text_x= 50
        Camera3.PlotProp.text_y= 30
        Camera3.Rays()
#        Camera3.LineIntegral(RR.Image, RR, plot=False) # True) #
#        Camera3.plot(ax1)
        iRay=0
        while iRay<Camera3.Nrays:
#            tmpRq, tmpTotalLength, tmpind =Camera3.CalculateLineIntegral(RR, iRay)
            tmpRq, tmpTotalLength, tmpind, tmpN =Camera3.CalculateLineIntegral(RR, iRay)#, Plot=True, ax1=ax1, iPlotRay=5, debug=False)
            Camera3.Sinogram[iRay]=tmpRq
            iRay=iRay+1
        tmpfn = fnLineIntegral+'Cam3.dat'
        print 'tmpfn', tmpfn
        Camera3.WriteLineIntegralToFile(tmpfn)

    if Cam4:
        theta1 = 60 - FOV/2
        theta2 = 60 + FOV/2
        Camera4=Camera.Camera(x=-300, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 4')
        Camera4.PlotProp.color='c'
        Camera4.PlotProp.text_x= -440
        Camera4.PlotProp.text_y= 30
        Camera4.Rays()
#        Camera4.LineIntegral(RR.Image, RR, plot=False) # True) #
        iRay=0
        while iRay<Camera4.Nrays:
            tmpRq, tmpTotalLength, tmpind, tmpN =Camera4.CalculateLineIntegral(RR, iRay)
            Camera4.Sinogram[iRay]=tmpRq
            iRay=iRay+1
        tmpfn = fnLineIntegral+'Cam4.dat'
        Camera4.WriteLineIntegralToFile(tmpfn)

    if Cam5:
        theta1 = 135 - FOV/2
        theta2 = 135 + FOV/2
        Camera5=Camera.Camera(x=500, z=0, theta1=theta1, theta2=theta2, Nrays=Nrays,Name='Cam 5')
        Camera5.PlotProp.color='m'
        Camera5.PlotProp.text_x= 500
        Camera5.PlotProp.text_y= 30
        Camera5.Rays()
#        Camera5.LineIntegral(RR.Image, RR, plot=False) # True) #
        iRay=0
        while iRay<Camera5.Nrays:
            tmpRq, tmpTotalLength, tmpind, tmpN =Camera5.CalculateLineIntegral(RR, iRay)
            Camera5.Sinogram[iRay]=tmpRq
            iRay=iRay+1
        tmpfn = fnLineIntegral+'Cam5.dat'
        Camera5.WriteLineIntegralToFile(tmpfn)
    #exit()

    ## Plot stuff follows

    if PlotType=='CameraPosition': 
        fig, (ax1) = plt.subplots(1, 1, figsize=(11, 4))
        fig.subplots_adjust(top=0.95, bottom=0.13, left=0.08, right=0.95)

        # generate 2 2d grids for the x & y bounds
#        z, x = np.mgrid[slice(RR.zmin+RR.dz/2, RR.zmax+RR.dz/2 , RR.dz),
        z, x = np.mgrid[slice(RR.zmax, RR.zmin-RR.dz/2 , -RR.dz),
                        slice(RR.xmin+RR.dx/2, RR.xmax+RR.dx/2 , RR.dx)]

        z_min, z_max = -np.abs(z).max(), np.abs(z).max()
        z_min, z_max = -np.abs(RR.Image).max(), np.abs(RR.Image).max()
        ax1.pcolor(x, z, RR.Image, cmap='RdBu', vmin=z_min, vmax=z_max)

        if Cam1: Camera1.plot(ax1)
        if Cam2: Camera2.plot(ax1)
        if Cam3: Camera3.plot(ax1)
        if Cam4: Camera4.plot(ax1)
        if Cam5: Camera5.plot(ax1)

    #plt.title('pcolor')
    # set the limits of the plot to the limits of the data
        ax1.axis([Domain.xmin+300, Domain.xmax-300, Domain.zmin, Domain.zmax])
    #plt.colorbar()

    #ax1.set_title("Original")
        ax1.set_xlabel("Distance (m)")
        ax1.set_ylabel("Heigth (m)")
        xmajorLocator   = MultipleLocator(500)
        xminorLocator   = MultipleLocator(20)
        ymajorLocator   = MultipleLocator(200)
        yminorLocator   = MultipleLocator(20)
        ax1.xaxis.set_major_locator(xmajorLocator)
        ax1.yaxis.set_major_locator(ymajorLocator)
        ax1.xaxis.set_minor_locator(xminorLocator)
        ax1.yaxis.set_minor_locator(yminorLocator)

    elif PlotType=='Target': 
        fig, (ax1) = plt.subplots(1, 1, figsize=(4, 4))
        fig.subplots_adjust(top=0.95, bottom=0.13, left=0.18, right=0.95)

        # generate 2 2d grids for the x & y bounds
#        z, x = np.mgrid[slice(RR.zmin, RR.zmax+RR.dz , RR.dz),
        z, x = np.mgrid[slice(RR.zmax, RR.zmin-RR.dz/2 , -RR.dz),
                        slice(RR.xmin, RR.xmax+RR.dx , RR.dx)]

    #    print z
    #    print x
    #    print RR.Image
    #    exit()
    #    z_min, z_max = -np.abs(z).max(), np.abs(z).max()
        z_min, z_max = -np.abs(RR.Image).max(), np.abs(RR.Image).max()
        ax1.pcolor(x, z, RR.Image, cmap='RdBu', vmin=z_min, vmax=z_max)
    #    print "image.max()", image.max()
    #    ax1.pcolor(x, z, image, cmap='jet', vmin=z_min, vmax=z_max)

        if Cam1: Camera1.plot(ax1)
        if Cam2: Camera2.plot(ax1)
        if Cam3: Camera3.plot(ax1)
        if Cam4: Camera4.plot(ax1)
        if Cam5: Camera5.plot(ax1)

        ax1.axis([RR.xmin-RR.dx/2, RR.xmax+RR.dx/2, RR.zmin-RR.dz/2, RR.zmax+RR.dz/2])

        ax1.set_xlabel("Distance (m)")
        ax1.set_ylabel("Heigth (m)")
        xmajorLocator   = MultipleLocator(100)
        xminorLocator   = MultipleLocator(10)
        ymajorLocator   = MultipleLocator(100)
        yminorLocator   = MultipleLocator(10)
        ax1.xaxis.set_major_locator(xmajorLocator)
        ax1.yaxis.set_major_locator(ymajorLocator)
        ax1.xaxis.set_minor_locator(xminorLocator)
        ax1.yaxis.set_minor_locator(yminorLocator)

    elif PlotType=='Sinogram':
        Sinogram = np.zeros((NCameras, len(Camera1.Sinogram)))
        Sinogram[0,:] = Camera1.Sinogram
        Sinogram[1,:] = Camera2.Sinogram
        Sinogram[2,:] = Camera3.Sinogram
        if Cam4: Sinogram[3,:] = Camera4.Sinogram
        if Cam5: Sinogram[4,:] = Camera5.Sinogram
    #    print Sinogram.shape
        fig, (ax1) = plt.subplots(1, 1, figsize=(4, 4))
        fig.subplots_adjust(bottom=0.13)
        ax1.imshow(Sinogram, cmap=plt.cm.Greys_r, aspect='auto',
                   extent=(0, Nrays, Sinogram.shape[0]+0.5, 0.5),
                   interpolation='none')
    #           extent=(0, 180, 0, sinogram.shape[0]), 

    #    xmajorLocator   = MultipleLocator(500)
    #    xminorLocator   = MultipleLocator(20)
        ymajorLocator   = MultipleLocator(1)
    #    yminorLocator   = MultipleLocator(20)
    #    ax1.xaxis.set_major_locator(xmajorLocator)
        ax1.yaxis.set_major_locator(ymajorLocator)
    #    ax1.xaxis.set_minor_locator(xminorLocator)
    #    ax1.yaxis.set_minor_locator(yminorLocator)
        ax1.set_xlabel("Viewing direction number")
        ax1.set_ylabel("Camera number")


    plt.show()

#else:

#parser = OptionParser(usage='Generate synthetic line integrals.')
#parser.add_option('--Test', dest='Test', default=None,
#                  help="""Name of test to be made. See also Tests.py.""") 
#parser.add_option('--force', dest='force', default=False,
#                  help="""Generate new test files in TestDir.""") 
#(args,rest)   =  parser.parse_args()#

#if args.Test != None:
#
#    TestDir = '../Tests/'

#    print '{:s} running Test: {:s}'.format( __file__ , args.Test)

#    RunTest(args.Test, TestDir, force)

